using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace WgSoup {
	public class SliderConstChange : MVRScript {

		// IMPORTANT - DO NOT make custom enums. The dynamic C# complier crashes Unity when it encounters these for
		// some reason
        protected JSONStorable sliderValue;
        protected JSONStorableFloat sliderMinJSON;
        protected JSONStorableFloat sliderMaxJSON;
        protected JSONStorableFloat sliderPushJSON;
        protected JSONStorableFloat sliderChangeJSON;
        protected JSONStorableFloat sliderQuickJSON;
        protected JSONStorableFloat currentValueJSON;
        protected JSONStorableFloat delayValueJSON;
        protected JSONStorableFloat sliderValueTarget;
        protected float timer=0;
        protected float pushInitialVal=0;
        protected float pushStartVal=0;
        protected float pushFinalVal=0;
        protected float pushDelay=0;
        protected float pushProgress=1;
        

        
		// IMPORTANT - DO NOT OVERRIDE Awake() as it is used internally by MVRScript - instead use Init() function which
		// is called right after creation
		public override void Init() {
			try {
                //UI creation
                if(containingAtom.type == "UISlider")
                {
                sliderMinJSON = new JSONStorableFloat("MIN Value", 0.0f, 0f, 1f, true, true);
                sliderMaxJSON = new JSONStorableFloat("MAX Value", 1f, 0f , 1f, true, true);
                sliderChangeJSON = new JSONStorableFloat("Constant Change", 0f, -1f, 1f, false, true);
                sliderPushJSON = new JSONStorableFloat("Increment", 0.1f, -1f, 1f, true, true);
                sliderQuickJSON = new JSONStorableFloat("Change Quickness", 2f, 0.1f, 10f, false, true);
                delayValueJSON = new JSONStorableFloat("PUSH Delay Value", 0f, 0f, 10f, false, true);
                currentValueJSON = new JSONStorableFloat("Current Slider Value (show)", 0f, 0f, 1f, true, true);
                sliderValueTarget = new JSONStorableFloat("Target Value", 0f, 0f, 1f, true, true);
                
                RegisterFloat(sliderMinJSON);
                CreateSlider(sliderMinJSON);
                RegisterFloat(sliderMaxJSON);
                CreateSlider(sliderMaxJSON);
                RegisterFloat(sliderChangeJSON);
                CreateSlider(sliderChangeJSON);
                RegisterFloat(sliderPushJSON);
                CreateSlider(sliderPushJSON);
                RegisterFloat(sliderQuickJSON);
                CreateSlider(sliderQuickJSON);
                RegisterFloat(delayValueJSON);
                CreateSlider(delayValueJSON);
                RegisterFloat(sliderValueTarget);
                CreateSlider(sliderValueTarget);
                CreateSlider(currentValueJSON);

				var Raise_Slider = CreateButton("PUSH value by increment", false);
                Raise_Slider.button.onClick.AddListener(Push_Slider_Foo);
                JSONStorableAction Raise_Slider_Action = new JSONStorableAction("PUSH value by increment", () => {Push_Slider_Foo();});
                RegisterAction(Raise_Slider_Action);
                JSONStorableAction Raise_Slider_Action_legacy = new JSONStorableAction("PUSH value", () => {Push_Slider_Foo();});
                RegisterAction(Raise_Slider_Action_legacy);

                var Raise_Target_Slider = CreateButton("PUSH value to target", false);
                Raise_Target_Slider.button.onClick.AddListener(Push_Slider_Target);
                JSONStorableAction Raise_Target_Slider_Action = new JSONStorableAction("PUSH value to target", () => {Push_Slider_Target();});
                RegisterAction(Raise_Target_Slider_Action);
                }

			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Start is called once before Update or FixedUpdate is called and after Init()
		void Start() {
			try {
                //Check if the plugin is assigned to a slider
                if(containingAtom.type != "UISlider")
                {
                    SuperController.LogError("Please attach to a slider trigger");
                }
                else
                {
                    sliderValue=containingAtom.GetStorableByID("Trigger");
                }
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Update is called with each rendered frame by Unity
		void Update() {

            try{
                if(containingAtom.type == "UISlider")
                {
                    //Avoid overlap between min and max value
                    if(sliderMinJSON.val>=sliderMaxJSON.val)
                    {
                        sliderMinJSON.val=sliderMaxJSON.val;
                    }
                    if(pushDelay>0) //If delay is set
                    {
                        pushDelay-=Time.deltaTime;
                    }//Push interpolation
                    else if(pushProgress<1.1f)
                    {
                        sliderValue.GetFloatJSONParam("value").val=Mathf.Lerp(pushStartVal, pushFinalVal, pushProgress);
                        pushProgress+=sliderQuickJSON.val/100;
                        //SuperController.LogMessage("pushProgress: "+pushProgress +" pushFinalVal: "+pushFinalVal+" StartVal: "+pushStartVal+" CurrentVal: "+sliderValue.GetFloatJSONParam("value").val);
                    }
                    //stop change at min
                    var sliderValueVal=sliderValue.GetFloatJSONParam("value").val;
                    if(sliderValueVal<=sliderMinJSON.val)
                    {
                        sliderValue.GetFloatJSONParam("value").val=sliderMinJSON.val;
                    }

                    //stop change at max and min
                    if(sliderValueVal>=sliderMaxJSON.val)
                    {
                        sliderValue.GetFloatJSONParam("value").val=sliderMaxJSON.val;
                    }
                    if(sliderValueVal<=sliderMinJSON.val)
                    {
                        sliderValue.GetFloatJSONParam("value").val=sliderMinJSON.val;
                    }
                    currentValueJSON.val=sliderValue.GetFloatJSONParam("value").val;
                    //Constant change                    
                    sliderValue.GetFloatJSONParam("value").val+=sliderChangeJSON.val/100;
                }
                
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// FixedUpdate is called with each physics simulation frame by Unity
		void FixedUpdate() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

        protected void Push_Slider_Foo()
        {
            pushDelay=delayValueJSON.val;
            pushFinalVal=sliderValue.GetFloatJSONParam("value").val+sliderPushJSON.val;
            pushStartVal=sliderValue.GetFloatJSONParam("value").val;
            pushProgress=0;
        }

        protected void Push_Slider_Target()
        {
            pushDelay=delayValueJSON.val;
            pushFinalVal=sliderValueTarget.val;
            pushStartVal=sliderValue.GetFloatJSONParam("value").val;
            pushProgress=0;
        }

		// OnDestroy is where you should put any cleanup
		// if you registered objects to supercontroller or atom, you should unregister them here
		void OnDestroy() {
		}

	}
}